/*! Tripetto Runner Foundation 6.0.5-4 - Copyright (C) 2021 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.functional.d.ts" />

declare module "@marblecore/hookup" {
    /** Explicit exports */
    export {
        Hooks,
        THooks,
        IHookPayload,
        HookProps,
        THookContext,
    } from "@marblecore/hookup/lib/hooks";
    export { IHookProperty } from "@marblecore/hookup/lib/property";
    /** Decorators */
    export { hookup } from "@marblecore/hookup/lib/decorator";
}

declare module "@marblecore/hookup/lib/hooks" {
    export interface IHookPayload<Name extends THooks> {
        hook: {
            name: Name;
            type: TType;
        };
    }
    export type THooks = string | void;
    export type HookProps<T extends IHookPayload<THooks>> = Pick<
        T,
        Exclude<keyof T, "hook">
    >;
    export type THookContext = Function | {};
    type TType = "synchronous" | "asynchronous" | "framed";
    type TFunc<
        Names extends THooks,
        Hook extends Names,
        Payload extends IHookPayload<Hook> = IHookPayload<Hook>
    > = (pPayload: Payload) => void;
    /**
     * Implements the hooks class.
     * @param Names Specifies the available hooks (specify using string literals).
     */
    export class Hooks<Names extends THooks> {
        /** Retrieves if the hooks buffer is enabled. */
        get isHookBufferEnabled(): boolean;
        /**
         * Register a hook.
         * @param sHook Specifies the hook name.
         * @param sType Specifies the hook type.
         * @param fnHook Specifies the hook function.
         * @param pContext Specifies the hook context.
         * @return Returns `this` to allow chaining.
         */
        hook<Hook extends Names, Payload extends IHookPayload<Hook>>(
            sHook: Hook,
            sType: TType,
            fnHook: TFunc<Names, Hook, Payload>,
            pContext?: THookContext
        ): this;
        /**
         * Register a grouped hook.
         * @param sHook Specifies the hook name.
         * @param sType Specifies the hook type.
         * @param fnHook Specifies the hook function.
         * @param pContext Specifies the hook context.
         * @return Returns `this` to allow chaining.
         */
        groupedHook<Hook extends Names, Payload extends IHookPayload<Hook>>(
            sHook: Hook,
            sType: TType,
            fnHook: TFunc<Names, Hook, Payload>,
            pContext?: THookContext
        ): this;
        /**
         * Emits a hook.
         * @param sHook Specifies the hook name.
         * @param pPayload Specifies the payload.
         * @param sMode Specifies the mode. Can be `fifo` or `lifo` (default is `fifo`).
         * @return Returns `true` if the hook is emitted.
         */
        emitHook<Hook extends Names, Payload extends IHookPayload<Hook>>(
            sHook: Hook,
            pPayload: HookProps<Payload>,
            sMode?: "fifo" | "lifo"
        ): boolean;
        /** Enables the hooks buffer. */
        bufferHooks(): void;
        /** Flushes the hooks buffer. */
        flushHooks(): void;
        /**
         * Removes a hook.
         * @param sHook Specifies the hook name.
         * @param pContextOrFunction Specifies the hook context or function to detach.
         * @return Returns `true` if the hook was found and removed.
         */
        unhook<Hook extends Names>(
            sHook: Hook,
            pContextOrFunction?: TFunc<Names, Hook> | THookContext
        ): boolean;
        /**
         * Unhooks all hooks from the specifies context.
         * @param pContext Specifies the context.
         * @return Returns `true` if one or more hooks were found and removed for
         * the specifies context.
         */
        unhookContext(pContext: THookContext): boolean;
        /** destroys the hooks (any pending hook emits are flushed synchronous). */
        destroy(): void;
    }
    export {};
}

declare module "@marblecore/hookup/lib/property" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup/lib/hooks";
    export interface IHookProperty<
        Name extends THooks,
        T extends {},
        K extends keyof T
    > extends IHookPayload<Name> {
        /** Name of the property. */
        property: K;
        /** Value of the property. */
        value: T[K];
    }
}

declare module "@marblecore/hookup/lib/decorator" {
    /** Dependencies */
    import { TAny } from "@marblecore/functional";
    import { Hooks, THooks } from "@marblecore/hookup/lib/hooks";
    /**
     * Binds a hook to a property.
     * @param sHook Specifies the hook to attach.
     */
    export function hookup<T extends Hooks<TAny>, Hook extends THooks>(
        sHook: Hook
    ): (
        pTarget: T,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ) => void;
}
